/**********************************************************************************************
    Copyright (C) 2006, 2007 Oliver Eichler oliver.eichler@gmx.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA

**********************************************************************************************/
#ifndef CGARMINWPTDB_H
#define CGARMINWPTDB_H

#include <QObject>
#include <QWidget>
#include <QMap>
#include <QPointer>

#include <IDevice.h>

#include "IProjection.h"

class QTabWidget;
class QString;
class CToolViewWpt;
class CGpx;
class QHttp;
class QProgressDialog;
class QHttpResponseHeader;

class CGarminDBWpt : public QObject
{
    Q_OBJECT
    public:
        CGarminDBWpt(QTabWidget * parent);
        virtual ~CGarminDBWpt();

        void gainFocus();

        struct Wpt_t : public Garmin::Wpt_t
        {
            Wpt_t();
            Wpt_t(const Garmin::Wpt_t& wpt);

            const Garmin::Wpt_t& toGarminWpt();

            QString ident;
            QString comment;
            QString facility;
            QString city;
            QString addr;
            QString crossroad;

            QPixmap icon;
            QString link;

            bool sticky;
        };

        /// receive waypoints from device
        void downloadWaypoints();
        /// send waypoints to device
        void uploadWaypoints();
        /// read access to database
        const QMap<QString,Wpt_t>& getWaypoints(){return waypoints;}
        /// create a new waypoint
        void newWaypoint();
        /// edit existing waypoint defined by key
        void editWaypoint(const QString& key);
        /// remove a waypoint by name
        void removeWaypoint(const QString& key);
        /// remove several waypoints by name
        void removeWaypoint(const QStringList& keys);
        /// load waypoints from GPX file
        void loadGPX(CGpx& gpx);
        /// save waypoints to GPX file
        void saveGPX(CGpx& gpx);
        /// remove all waypoints from the list
        void clear();
        /// get a list of geo caching waypoints from www.opencaching.de
        void queryOpenChachingDe(double lon, double lat, double radius);

        /// get iterator access to track point list
        QMap<QString,Wpt_t>::iterator begin(){return waypoints.begin();}
        /// get iterator access to track point list
        QMap<QString,Wpt_t>::iterator end(){return waypoints.end();}

        void removeWaypointBy();

    signals:
        void sigWaypointListChanged();

    protected slots:
        void slotRequestStarted(int );
        void slotRequestFinished(int , bool error);
        void slotDataReadProgress(int done, int total);
    protected:
        QPointer<QTabWidget> tab;

        /// tool view widget to display database information
        CToolViewWpt * toolview;

        /// the waypoint database
        QMap<QString,Wpt_t> waypoints;

        /// request protocol handler for waypoint queries
        QPointer<QHttp> opencaching;
        /// the progress dialog to inform about the state of the waypoint query
        QPointer<QProgressDialog> progressDlg;
        /// internal flag used by slotRequestFinished()
        bool rcvData;
        /// radius used by geo cache query (used to fine tune results)
        double radius;
        /// center point of geo cache query (used to fine tune results)
        XY center;
};

#endif //CGARMINWPTDB_H

