/**********************************************************************************************
    Copyright (C) 2007 Oliver Eichler oliver.eichler@gmx.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA

  Garmin and MapSource are registered trademarks or trademarks of Garmin Ltd.
  or one of its subsidiaries.

**********************************************************************************************/
#ifndef GARMIN_H
#define GARMIN_H

#include <stdint.h>

namespace Garmin
{
#pragma pack(1)
    struct Product_Data_t
    {
        uint16_t product_id;
        int16_t  software_version;
        char     str[];
    };

    struct Protocol_Data_t
    {
        uint8_t  tag;
        uint16_t data;
    };

    struct D108_Wpt_t
    {
        uint8_t  wpt_class;
        uint8_t  color;
        uint8_t  dspl;
        uint8_t  attr;
        uint16_t smbl;
        uint8_t  subclass[18];
        int32_t  lat;
        int32_t  lon;
        float    alt;
        float    dpth;
        float    dist;
        char     state[2];
        char     cc[2];
        char     str[];
    };

    struct D109_Wpt_t 
    {
        uint8_t  dtyp;
        uint8_t  wpt_class;
        uint8_t  dspl_color;
        uint8_t  attr;
        uint16_t smbl;
        uint8_t  subclass[18];
        int32_t  lat;
        int32_t  lon;
        float    alt;
        float    dpth;
        float    dist;
        char     state[2];
        char     cc[2];
        uint32_t ete;
        char     str[];
    };

    struct D110_Wpt_t
    {
        uint8_t  dtyp;
        uint8_t  wpt_class;
        uint8_t  dspl_color;
        uint8_t  attr;
        uint16_t smbl;
        uint8_t  subclass[18];
        int32_t  lat;
        int32_t  lon;
        float    alt;
        float    dpth;
        float    dist;
        char     state[2];
        char     cc[2];
        uint32_t ete;
        float    temp;
        uint32_t time;
        uint16_t wpt_cat;
        char     str[];
    };

    struct D301_Trk_t 
    {
        int32_t  lat;
        int32_t  lon;
        uint32_t time;
        float    alt;
        float    dpth;
        uint8_t  new_trk;
    };

    struct D302_Trk_t
    {
        int32_t  lat;
        int32_t  lon;
        uint32_t time;
        float    alt;
        float    dpth;
        float    temp;
        uint8_t  new_trk;
    };

// same as D312, but without color=16=transparent
    struct D310_Trk_Hdr_t
    {
        uint8_t  dspl;
        uint8_t  color;
        char     ident[];
    };

    struct D312_Trk_Hdr_t
    {
        uint8_t  dspl;
        uint8_t  color;
        char     ident[];
    };

    struct Map_Request_t
    {
        uint32_t dummy1;
        uint16_t dummy2;
        char     section[13];
    };

    struct Map_Info_t
    {
        uint8_t  tok;                ///< should be 0x4c
        uint16_t size;               ///< total entry size will be size + sizeof(tok) + sizeof(size)
        uint16_t product;            ///< product code as it can be found in the registry
        uint16_t dummy;              ///< country / char. set?
        uint32_t mapId;              ///< same id as in tdb and map filename
        char     name1[];
        /*
      There are several 0 terminated strings:
      "map name"
      "tile name"
      */
    };
    /*
    quint32 ???                 ///< the map id as it is used in the FAT table
    quint32 ???                 ///< always 0x00000000 (end token?)
    */


#pragma pack()

    enum serial_e
    {
         Pid_Ack_Byte           = 6
        ,Pid_Nak_Byte           = 21
    };

    enum L000_e
    {
         Pid_Protocol_Array     = 253
        ,Pid_Product_Rqst       = 254
        ,Pid_Product_Data       = 255
        ,Pid_Ext_Product_Data   = 248
    };

    enum L001_e
    {
         Pid_Command_Data       = 10
        ,Pid_Xfer_Cmplt         = 12
        ,Pid_Prx_Wpt_Data       = 19
        ,Pid_Wpt_Data           = 35
        ,Pid_Records            = 27
        ,Pid_Trk_Data           = 34
        ,Pid_Capacity_Data      = 95    //????
        ,Pid_Trk_Hdr            = 99
        ,Pid_Tx_Unlock_Key      = 108   //????
        ,Pid_Ack_Unlock_key     = 109   //????
    };

    enum A010_e
    {
         Cmnd_Abort_Transfer    = 0
        ,Cmnd_Transfer_Prx      = 3
        ,Cmnd_Transfer_Trk      = 6
        ,Cmnd_Transfer_Wpt      = 7
        ,Cmnd_Transfer_Mem      = 63


    };

    struct Wpt_t;

    extern void operator<<(Wpt_t& tar, const D108_Wpt_t& src);
    extern int  operator>>(const Wpt_t& src, D108_Wpt_t& tar);

    extern void operator<<(Wpt_t& tar, const D109_Wpt_t& src);
    extern int  operator>>(const Wpt_t& src, D109_Wpt_t& tar);

    extern void operator<<(Wpt_t& tar, const D110_Wpt_t& src);
    extern int  operator>>(const Wpt_t& src, D110_Wpt_t& tar);

    struct Track_t;
    struct TrkPt_t;

    extern void operator<<(Track_t& tar, const D310_Trk_Hdr_t& src);
    extern void operator<<(Track_t& tar, const D312_Trk_Hdr_t& src);
    //extern int  operator>>(const Track_t& src, D312_Trk_Hdr_t& tar);

    extern void operator<<(TrkPt_t& tar, const D301_Trk_t& src);
    extern void operator<<(TrkPt_t& tar, const D302_Trk_t& src);

    //extern int  operator>>(const TrkPt_t& src, D302_Trk_t& tar);

}



#endif //GARMIN_H


