/**********************************************************************************************
    Copyright (C) 2007 Oliver Eichler oliver.eichler@gmx.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA

  Garmin and MapSource are registered trademarks or trademarks of Garmin Ltd.
  or one of its subsidiaries.

**********************************************************************************************/
#ifndef IDEVICEDEFAULT_H
#define IDEVICEDEFAULT_H

#include "IDevice.h"

namespace Garmin
{
    /// default implementation of a device driver
    /**
        Please refer to IDevice for documentation on the public interface.
        The default implementation of all methods is to throw an exception,
        to signal that a real implementation is missing. If your device lacks
        a ceratin feature simply do not implement the corresponding '_' method.
    */
    class IDeviceDefault : public IDevice
    {
    public:
        IDeviceDefault();
        virtual ~IDeviceDefault();
        /// see IDevice::uploadMap()
        virtual void uploadMap(const uint8_t * mapdata, uint32_t size, const char * key, void (*callback)(uint32_t,uint32_t,void*), void* data);
        /// see IDevice::queryMap()
        virtual void queryMap(std::list<Map_t>& maps);
        /// see IDevice::downloadWaypoints()
        virtual void downloadWaypoints(std::list<Garmin::Wpt_t>& waypoints);
        /// see IDevice::uploadWaypoints()
        virtual void uploadWaypoints(std::list<Garmin::Wpt_t>& waypoints);
        /// see IDevice::downloadTracks()
        virtual void downloadTracks(std::list<Garmin::Track_t>& tracks);
        /// see IDevice::getCopyright()
        virtual const std::string& getCopyright();
        /// see IDevice::getLastError()
        virtual const std::string& getLastError();
        /// see IDevice::setPort()
        virtual void setPort(const char * p);

    protected:
        virtual void _acquire() = 0;

        virtual void _uploadMap(const uint8_t * mapdata, uint32_t size, const char * key, void (*callback)(uint32_t,uint32_t,void*), void* data);
        virtual void _queryMap(std::list<Map_t>& maps);
        virtual void _downloadWaypoints(std::list<Garmin::Wpt_t>& waypoints);
        virtual void _uploadWaypoints(std::list<Garmin::Wpt_t>& waypoints);
        virtual void _downloadTracks(std::list<Garmin::Track_t>& tracks);

        virtual void _release() = 0;

        std::string copyright;

        std::string lasterror;

        std::string port;

    };
}
#endif //IDEVICEDEFAULT_H

